/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package worlds3dclient;

import java.awt.event.KeyEvent;
import java.util.HashMap;

/**
 *
 * @author eduseiti
 */
public class CommandProcessor { 
    
    public interface CommandExecutedOperation {
        public void execute(int whichCommand);
    }
    
    
    private class HandledCommand {
        private String commandText;
        CommandExecutedOperation operation;
        
        public HandledCommand(String whichCommandText, CommandExecutedOperation whichOperation) {
            commandText = whichCommandText;
            operation   = whichOperation;
        }
    }
        
    private static CommandProcessor runningInstance = null;
    
    private static final SystemFinalize systemFinalize;
    
    private static final ServerSender serverSender;
    
    private static final HashMap<Integer, CommandExecutedOperation> handledCommands;
    
    static {
        handledCommands = new HashMap<>();

        systemFinalize = new SystemFinalize();
        
        // serverSender = ServerSender.getInstance("192.168.0.199", 4011);
        serverSender = ServerSender.getInstance();
        
        handledCommands.put(ServerSender.CREATE_CREATURE, serverSender);
        handledCommands.put(KeyEvent.VK_UP, serverSender);
        handledCommands.put(KeyEvent.VK_DOWN, serverSender);
        handledCommands.put(KeyEvent.VK_LEFT, serverSender);
        handledCommands.put(KeyEvent.VK_RIGHT, serverSender);
        handledCommands.put(KeyEvent.VK_F1, serverSender);
        handledCommands.put(KeyEvent.VK_F2, serverSender);
        handledCommands.put(KeyEvent.VK_R, serverSender);
        handledCommands.put(KeyEvent.VK_SPACE, serverSender);
        handledCommands.put(KeyEvent.VK_ESCAPE, systemFinalize);        
    };
    
    
    public static CommandProcessor getInstance() {
        if(runningInstance == null) {
            runningInstance = new CommandProcessor();
        }
        
        return(runningInstance);
    }
    
    private CommandProcessor() {
    }
    
    public void process(int receivedCommand) {
        CommandExecutedOperation operation = null;
        
        if((operation = handledCommands.get(receivedCommand)) != null) {
            operation.execute(receivedCommand);
        }
    }
    
    public void finalize() {
        serverSender.finalize();
    }
}
