/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package worlds3dclient;

import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Vector;


/**
 *
 * @author eduseiti
 */
public class ServerSender implements CommandProcessor.CommandExecutedOperation {
    
    private static final String DEFAULT_SERVER = "localhost";
    private static final int DEFAULT_PORT = 4011;
    
    private static final int MAXIMUM_RETRIES = 1000;
    
    public static final int CREATE_CREATURE = 0x1001;
    
    private static final String ATTACH_CAMERA = "camera %d";
    
    private static final String START_ENGINE = "start %d";
    private static final String STOP_ENGINE  = "stop %d";
    
    private static final String SET_TURN = "setTurn %d %.2f 0 0";
    
    private static final String UPDATE_CREATURE_INFO = "getfullstatus3D %d";
    
    private static final String SHOW_WORLD_THINGS = "showworldthings";
    
    private static final float RADIUS_45  = 0.783f;
    private static final float RADIUS_315 = 5.491f;
    
    private static final float DELTA_SPEED = 0.1f;

    
    private String serverName = null;
    private int serverPort;
    
    private Socket socket = null;
    private PrintWriter dataSending = null;
    private BufferedReader dataReceiving = null;
    
    private static ServerSender runningInstance = null;
    
    private static final HashMap<Integer, String> commandsTranslationMap;
    
    
    private Creature creature;
    
    
    static {
        commandsTranslationMap = new HashMap<>();
        
        commandsTranslationMap.put(KeyEvent.VK_F1, "showworldthings");
        commandsTranslationMap.put(KeyEvent.VK_F2, "new %.6f %.6f 0");

        commandsTranslationMap.put(CREATE_CREATURE, "new %.6f %.6f 0");

        commandsTranslationMap.put(KeyEvent.VK_R, "refuel %d");

        commandsTranslationMap.put(KeyEvent.VK_UP, "setTurn %d %.2f 0 0");
        commandsTranslationMap.put(KeyEvent.VK_DOWN, "setTurn %d %.2f 0 0");
        commandsTranslationMap.put(KeyEvent.VK_SPACE, "setTurn %d 0 0 0");
        commandsTranslationMap.put(KeyEvent.VK_LEFT, "setAngle %d 0 0 %.3f");
        commandsTranslationMap.put(KeyEvent.VK_RIGHT, "setAngle %d 0 0 %.3f");
    };
    

    public static ServerSender getInstance() {
        if(runningInstance == null) {
            runningInstance = new ServerSender(DEFAULT_SERVER, DEFAULT_PORT);
        }
        
        return(runningInstance);
    }

    
    
    public static ServerSender getInstance(String whichServerName, int whichServerPort) {
        if(runningInstance == null) {
            runningInstance = new ServerSender(whichServerName, whichServerPort);
        }
        
        return(runningInstance);
    }
    
       
    private ServerSender(String whichServerName, int whichServerPort) {
        serverName = whichServerName;
        serverPort = whichServerPort;
        
        try {
            socket = new Socket(serverName, serverPort);
            
            dataSending = new PrintWriter(socket.getOutputStream(), true);
            
            dataReceiving = new BufferedReader(new InputStreamReader(socket.getInputStream()));
            
            
            // Get server welcome message
            
            String serverResponse = null;
            
            serverResponse = dataReceiving.readLine();
            System.out.println(serverResponse);
            
            while(dataReceiving.ready() == true) {
                serverResponse = dataReceiving.readLine();
                System.out.println(serverResponse);
            }
            
        } catch (UnknownHostException e) {
            System.err.println("Could not find server " + serverName);
            System.exit(-1);
        } catch (IOException e) {
            System.err.println("Could not get IO for the connection to " + serverName + ":" + serverPort);
            System.exit(-1);            
        }
    }
      
    public void finalize() {
        dataSending.close();
        
        try {
            dataReceiving.close();
            socket.close();
        } catch (IOException e) {
            System.err.println("Error when finalizing ServerSender connections.");            
        }
    }
    
    
    
    private Vector<String> sendCommand (String commandLine) {
        
        Vector<String> returnedData;
        
        returnedData = new Vector<>();
        
        
        try {
            String commandResponse;
            
            System.out.println("sendCommand: " + commandLine);
            dataSending.println(commandLine);
            dataSending.flush();

            World3D_client.commandResponses.append("\n" + commandLine + "\n");

            if(dataReceiving.ready() != true) {
                try {
                    Thread.sleep(500);
                    World3D_client.commandResponses.append("Waiting a little bit for the server answer to become available...\n");
                    System.err.println("Waiting a little bit for the server answer to become available...\n");
                } catch (Exception e) {
                    System.err.println(e);
                }
            }
            
            do {
                commandResponse = dataReceiving.readLine();
                
                if((commandResponse.equals("") == true) && (dataReceiving.ready() == true)){
                    commandResponse = dataReceiving.readLine();                
                }
                
                returnedData.add(commandResponse);
                System.out.println(commandResponse);
                World3D_client.commandResponses.append(commandResponse + "\n");
            } while(dataReceiving.ready() == true);

        } catch (IOException e) {
            System.err.println("Error receiving server response for \"" + commandLine + "\"");
        }
        
        return(returnedData);
    }
    
    
    public void execute(int whichCommand) {
        
        Vector<String> response;
        String serverCommand = null;
        
        if((serverCommand = commandsTranslationMap.get(whichCommand)) != null) {
            
            float pitchValue = -1;
            float speedValue = -1;
            
            switch(whichCommand) {
                case KeyEvent.VK_F1: {
                    sendCommand(serverCommand);
                    
                    break;
                }
                case CREATE_CREATURE: 
                case KeyEvent.VK_F2: {
                    creature = new Creature();
                    
                    sendCommand(String.format(Locale.US, serverCommand, creature.getX(), creature.getY()));
                                        
                    response = sendCommand(SHOW_WORLD_THINGS);
                    
                    int lastID = -1;
                    
                    for(int i = 1; i < response.size(); i++) {
                        String[] parsedResponse = response.get(i).split("[ | :]");
                        
                        if(parsedResponse[0].equals("Creature") == true) {
                            lastID = Integer.valueOf(parsedResponse[1]);
                        } else {
                            break;
                        }
                            
                    } 
                    
                    creature.setID(lastID);
                    
                    sendCommand(String.format(Locale.US, ATTACH_CAMERA, creature.getID()));

                    sendCommand(String.format(Locale.US, START_ENGINE, creature.getID()));
                                        
                    break;
                }
                    
                case KeyEvent.VK_RIGHT: {
                    if(creature == null) {
                        break;
                    } else {
                        pitchValue = creature.getPitch() + RADIUS_45;
                    }
                }
                    
                case KeyEvent.VK_LEFT: {
                    if(pitchValue == -1) {
                        if(creature == null) {
                            break;
                        } else {
                            pitchValue = creature.getPitch() - RADIUS_45;
                        }
                    }
                    
                    if(pitchValue < 0) {
                        if(pitchValue <= -RADIUS_45) {
                            pitchValue = RADIUS_315;                            
                        } else {
                            pitchValue = 0;
                        }
                    } else {
                        if(pitchValue > RADIUS_315) {
                            pitchValue = 0;
                        }
                    }

                    // Turn to the defined angle
                    
                    sendCommand(String.format(Locale.US, serverCommand, creature.getID(), pitchValue));

                    // Return to the current linear speed
                    
                    sendCommand(String.format(Locale.US, SET_TURN, creature.getID(), creature.getLinearSpeed()));

                    break;
                }

                case KeyEvent.VK_UP: {
                    if(creature == null) {
                        break;
                    } else {
                        speedValue = creature.getLinearSpeed() + DELTA_SPEED;
                    }
                }
                    
                case KeyEvent.VK_DOWN: {
                    if(speedValue == -1) {
                        if((creature == null) || (creature.getLinearSpeed() == 0)) {
                            break;
                        } else {
                            speedValue = creature.getLinearSpeed() - DELTA_SPEED;
                        }
                    }
                    
                    sendCommand(String.format(Locale.US, serverCommand, creature.getID(), speedValue));
                    
                    break;
                }
                    
                case KeyEvent.VK_R: {
                    
                    sendCommand(String.format(Locale.US, serverCommand, creature.getID()));                    

                    break;
                }
                    
                case KeyEvent.VK_SPACE: {
                    
                    sendCommand(String.format(Locale.US, serverCommand, creature.getID()));                    

                    break;
                }
            }
        
            
            if (creature != null) {
                response = sendCommand(String.format(Locale.US, UPDATE_CREATURE_INFO, creature.getID()));

                if(response.get(0).equals("") == true) {
                    System.out.println("Empty answer from server...\n");
                    World3D_client.commandResponses.append("Empty answer from server...\n");                
                } else {                
                    creature.updateStatus(response.get(0));
                }
            }
        }
    }
}
